import React, { useContext, useRef, useEffect } from 'react';
import { createTheme as createTheme$1, styleMap } from 'treat';
import { TreatProvider, useStyles } from 'react-treat';
import hash from '@emotion/hash';

let instance = console;
function setInstance(logger) {
    instance = logger;
}
// TODO: Use `Set` when no polyfill is required for it anymore
const printedMessages = new Map();
function warnOnce(message) {
    if (!printedMessages.has(message)) {
        instance.warn(message);
        printedMessages.set(message, 1);
    }
}

// eslint-disable-next-line import/prefer-default-export
function modularScale(ratio, steps = [-1, -0.5, 0, +1, +2, +3, +4, +5, +6]) {
    const result = {};
    steps.forEach((step) => {
        // Rounding to 3 decimal places
        result[step] = `${Math.round(ratio ** step * 1e3) / 1e3}rem`;
    });
    return result;
}

const isDev = process.env.NODE_ENV !== 'production';
const canUseDOM = typeof window !== 'undefined';

class NullRuleManager {
    // eslint-disable-next-line class-methods-use-this
    increaseUsage() {
        if (isDev) {
            // TODO: Add instructions for resolving the situation
            if (canUseDOM) {
                warnOnce('Client-side rendering of dynamic styles is not set up');
            }
            else {
                warnOnce('Server-side rendering of dynamic styles is not set up');
            }
        }
    }
    // eslint-disable-next-line class-methods-use-this
    decreaseUsage() { }
}
class OptimizedRuleManager {
    constructor(injector, initialRuleIndexesByClassName) {
        this.ruleIndexesByClassName = new Map();
        this.usageCountsByClassName = new Map();
        this.injector = injector;
        this.ruleIndexesByClassName = initialRuleIndexesByClassName;
    }
    increaseUsage(className, cssText) {
        const prevUsageCount = this.usageCountsByClassName.get(className) || 0;
        this.usageCountsByClassName.set(className, prevUsageCount + 1);
        // Append new rule only if it wasn't available in the server-rendered code
        if (!prevUsageCount && !this.ruleIndexesByClassName.has(className)) {
            this.ruleIndexesByClassName.set(className, this.injector.addRule(cssText()));
        }
    }
    decreaseUsage(className, byAmount) {
        const nextUsageCount = 
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        this.usageCountsByClassName.get(className) - byAmount;
        if (nextUsageCount) {
            this.usageCountsByClassName.set(className, nextUsageCount);
        }
        else {
            this.usageCountsByClassName.delete(className);
            // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
            this.injector.nullifyRule(this.ruleIndexesByClassName.get(className));
            this.ruleIndexesByClassName.delete(className);
        }
    }
}

/*
Based on:
- https://github.com/threepointone/glamor/blob/78ffd84bfba187f4037a29b389e5a349dbb6e9a8/packages/glamor/src/StyleSheet.ts
- https://github.com/styled-components/styled-components/blob/94df8bb57c8ba01cd33268b65df93499a7961cb2/packages/styled-components/src/sheet/Tag.js
*/
const MAX_RULE_COUNT = 0xffff;
function createAndMountStyleElement() {
    const el = document.createElement('style');
    el.dataset.glaze = '';
    return document.head.appendChild(el);
}
function getSheet() {
    // Hydrate existing style node if available
    for (let i = 0, { length } = document.styleSheets; i < length; i += 1) {
        const styleSheet = document.styleSheets[i];
        if (styleSheet.ownerNode.dataset.glaze === '') {
            return styleSheet;
        }
    }
    const styleEl = createAndMountStyleElement();
    // Avoid Edge bug where empty style elements don't create sheets
    styleEl.appendChild(document.createTextNode(''));
    // Avoid Firefox quirk where the style element might not have a sheet property
    return styleEl.sheet || getSheet();
}
function getInitialRuleIndexes(rules) {
    const ruleIndexesByClassName = new Map();
    for (let i = 0, { length } = rules; i < length; i += 1) {
        const rule = rules[i];
        if (rule.type === CSSRule.STYLE_RULE) {
            // Remove leading '.' from class selector
            const className = rule.selectorText.slice(1);
            ruleIndexesByClassName.set(className, i);
        }
        // TODO: Add support for `CSSMediaRule`
    }
    return ruleIndexesByClassName;
}
class NullStyleInjector {
    constructor() {
        this.ruleManager = new NullRuleManager();
    }
    // eslint-disable-next-line class-methods-use-this
    addRule() {
        return 0;
    }
    // eslint-disable-next-line class-methods-use-this
    nullifyRule() { }
}
class VirtualStyleInjector {
    constructor() {
        this.ruleManager = new OptimizedRuleManager(this, new Map());
        this.cssTexts = [];
    }
    getStyleElement() {
        return (React.createElement("style", { "data-glaze": "", 
            // eslint-disable-next-line react/no-danger
            dangerouslySetInnerHTML: { __html: this.cssTexts.join('') } }));
    }
    addRule(cssText) {
        return this.cssTexts.push(cssText) - 1;
    }
    // No styles are revoked during SSR
    // eslint-disable-next-line class-methods-use-this
    nullifyRule() { }
}
class OptimizedStyleInjector {
    constructor() {
        this.sheet = getSheet();
        this.ruleCount = 0;
        this.freeIndexes = [];
        this.ruleManager = new OptimizedRuleManager(this, getInitialRuleIndexes(this.sheet.rules));
    }
    addRule(cssText) {
        if (this.freeIndexes.length) {
            // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
            return this.replaceRule(this.freeIndexes.pop(), cssText);
        }
        // eslint-disable-next-line no-plusplus
        return this.sheet.insertRule(cssText, this.ruleCount++);
    }
    nullifyRule(index) {
        if (index === this.ruleCount - 1) {
            // eslint-disable-next-line no-plusplus
            this.sheet.deleteRule(--this.ruleCount);
        }
        else {
            // Only allow replacements to prevent modification of existing indexes
            const dummyRule = '#_{}';
            this.freeIndexes.push(this.replaceRule(index, dummyRule));
        }
    }
    replaceRule(index, cssText) {
        this.sheet.deleteRule(index);
        return this.sheet.insertRule(cssText, index);
    }
}
class DebuggableStyleInjector {
    constructor() {
        this.ruleManager = new OptimizedRuleManager(this, new Map());
        this.nodes = [];
        this.freeIndexes = [];
        this.styleEl = createAndMountStyleElement();
    }
    addRule(cssText) {
        if (this.freeIndexes.length) {
            // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
            return this.replaceRule(this.freeIndexes.pop(), cssText);
        }
        const index = this.nodes.length;
        if (index === MAX_RULE_COUNT) {
            warnOnce('An excessive amount of styling rules have been injected at runtime. Consider using inline styles or CSS variables instead.');
        }
        const node = document.createTextNode(cssText);
        this.styleEl.appendChild(node);
        this.nodes[index] = node;
        return index;
    }
    nullifyRule(index) {
        if (index === this.nodes.length - 1) {
            // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
            this.styleEl.removeChild(this.nodes.pop());
        }
        else {
            // Only allow replacements to prevent modification of existing indexes
            const dummyRule = '';
            this.freeIndexes.push(this.replaceRule(index, dummyRule));
        }
    }
    replaceRule(index, cssText) {
        this.nodes[index].textContent = cssText;
        return index;
    }
}

const StyleInjectorContext = React.createContext(new NullStyleInjector());
function StyleInjectorProvider({ children, injector = isDev
    ? new DebuggableStyleInjector()
    : new OptimizedStyleInjector(), }) {
    return (React.createElement(StyleInjectorContext.Provider, { value: injector }, children));
}

function createTheme(tokens, localDebugName) {
    const { breakpoints, shorthands, aliases } = tokens;
    return {
        ref: createTheme$1(tokens, localDebugName),
        breakpoints,
        shorthands,
        aliases,
    };
}
// TODO: symmetricScale()
const spacing = {
    0: 0,
    '1px': 1,
    1: '.25rem',
    2: '.5rem',
    3: '.75rem',
    4: '1rem',
    5: '1.25rem',
    6: '1.5rem',
    8: '2rem',
    10: '2.5rem',
    12: '3rem',
    16: '4rem',
    20: '5rem',
    24: '6rem',
    32: '8rem',
    40: '10rem',
    48: '12rem',
    56: '14rem',
    64: '16rem',
};
const defaultTheme = {
    breakpoints: [640, 768, 1024, 1280],
    scales: {
        spacing,
        size: {
            ...spacing,
            auto: 'auto',
            '100%': '100%',
        },
        fontSize: modularScale(1.333),
        lineHeight: {
            1: 1,
            tight: 1.25,
            snug: 1.375,
            base: 1.5,
            relaxed: 1.625,
            loose: 2,
        },
        letterSpacing: {
            wide: '.025em',
        },
        borderWidth: { 1: 1, 2: 2, 4: 4, 8: 8 },
        radius: {
            sm: '.125rem',
            md: '.25rem',
            lg: '.5rem',
            full: 9999,
        },
        shadow: {
            sm: '0 1px 3px 0 rgba(0, 0, 0, 0.1), 0 1px 2px 0 rgba(0, 0, 0, 0.06)',
            md: '0 4px 6px -1px rgba(0, 0, 0, 0.1), 0 2px 4px -1px rgba(0, 0, 0, 0.06)',
            lg: '0 10px 15px -3px rgba(0, 0, 0, 0.1), 0 4px 6px -2px rgba(0, 0, 0, 0.05)',
            xl: '0 20px 25px -5px rgba(0, 0, 0, 0.1), 0 10px 10px -5px rgba(0, 0, 0, 0.04)',
            inner: 'inset 0 2px 4px 0 rgba(0, 0, 0, 0.06)',
            outline: '0 0 0 3px rgba(66, 153, 225, 0.5)',
        },
    },
    shorthands: {
        // TODO: Remove if widely supported by browsers
        inset: ['top', 'right', 'bottom', 'left'],
        insetX: ['left', 'right'],
        insetY: ['top', 'bottom'],
        // TODO: Remove if widely supported by browsers
        size: ['width', 'height'],
        paddingX: ['paddingLeft', 'paddingRight'],
        paddingY: ['paddingTop', 'paddingBottom'],
        marginX: ['marginLeft', 'marginRight'],
        marginY: ['marginTop', 'marginBottom'],
    },
    aliases: {
        p: 'padding',
        px: 'paddingX',
        py: 'paddingY',
        pt: 'paddingTop',
        pr: 'paddingRight',
        pb: 'paddingBottom',
        pl: 'paddingLeft',
        // TODO: For logical properties, e.g. lpi, lpis, lpie, lpb, lpbs, lpbe
        m: 'margin',
        mx: 'marginX',
        my: 'marginY',
        mt: 'marginTop',
        mr: 'marginRight',
        mb: 'marginBottom',
        ml: 'marginLeft',
        // TODO: For logical properties, e.g. lmi, lmis, lmie, lmb, lmbs, lmbe
        bg: 'background',
    },
    resolvers: {
        top: 'spacing',
        right: 'spacing',
        bottom: 'spacing',
        left: 'spacing',
        zIndex: 'zIndex',
        flexBasis: 'size',
        gridGap: 'spacing',
        gridRowGap: 'spacing',
        gridColumnGap: 'spacing',
        gap: 'spacing',
        rowGap: 'spacing',
        columnGap: 'spacing',
        width: 'size',
        minWidth: 'size',
        maxWidth: 'size',
        height: 'size',
        minHeight: 'size',
        maxHeight: 'size',
        padding: 'spacing',
        // TODO: paddingInline: 'spacing',
        // TODO: paddingInlineStart: 'spacing',
        // TODO: paddingInlineEnd: 'spacing',
        // TODO: paddingBlock: 'spacing',
        // TODO: paddingBlockStart: 'spacing',
        // TODO: paddingBlockEnd: 'spacing',
        paddingTop: 'spacing',
        paddingRight: 'spacing',
        paddingBottom: 'spacing',
        paddingLeft: 'spacing',
        margin: 'spacing',
        // TODO: marginInline: 'spacing',
        // TODO: marginInlineStart: 'spacing',
        // TODO: marginInlineEnd: 'spacing',
        // TODO: marginBlock: 'spacing',
        // TODO: marginBlockStart: 'spacing',
        // TODO: marginBlockEnd: 'spacing',
        marginTop: 'spacing',
        marginRight: 'spacing',
        marginBottom: 'spacing',
        marginLeft: 'spacing',
        fontFamily: 'fontFamily',
        fontSize: 'fontSize',
        fontWeight: 'fontWeight',
        lineHeight: 'lineHeight',
        color: 'color',
        textShadow: 'shadow',
        letterSpacing: 'letterSpacing',
        background: 'color',
        backgroundColor: 'color',
        border: 'border',
        borderTop: 'border',
        borderRight: 'border',
        borderBottom: 'border',
        borderLeft: 'border',
        borderColor: 'color',
        borderTopColor: 'color',
        borderRightColor: 'color',
        borderBottomColor: 'color',
        borderLeftColor: 'color',
        borderWidth: 'borderWidth',
        borderTopWidth: 'borderWidth',
        borderRightWidth: 'borderWidth',
        borderBottomWidth: 'borderWidth',
        borderLeftWidth: 'borderWidth',
        borderRadius: 'radius',
        borderTopLeftRadius: 'radius',
        borderTopRightRadius: 'radius',
        borderBottomRightRadius: 'radius',
        borderBottomLeftRadius: 'radius',
        outlineColor: 'color',
        boxShadow: 'shadow',
        opacity: 'opacity',
    },
};

const ThemeContext = React.createContext(undefined);
function ThemeProvider({ theme, children, }) {
    // TODO: Try augmenting server-rendered dynamic styles or rehydrate them
    return (React.createElement(TreatProvider
    // Show a clear error message during runtime, even if `theme` is nullish
    , { 
        // Show a clear error message during runtime, even if `theme` is nullish
        theme: theme === null || theme === void 0 ? void 0 : theme.ref },
        React.createElement(ThemeContext.Provider, { value: theme }, children)));
}
function useTheme() {
    return useContext(ThemeContext);
}

var styleRefs = styleMap((theme) => {
    const result = {};
    Object.entries(theme.resolvers).forEach(([property, scale]) => {
        if (scale) {
            const tokens = Object.entries(theme.scales[scale] || {});
            tokens.forEach(([key, value]) => {
                result[`${property}-${key}`] = { [property]: value };
            });
        }
    });
    return result;
});

/* Prefer performance over elegance, as this code is critical for the runtime */
function kebabCaseReplacer(match) {
    return `-${match.toLowerCase()}`;
}
function getClassName(identName) {
    return isDev ? `DYNAMIC_${identName}` : `d_${hash(identName)}`;
}
function useStyling() {
    const staticClassNames = useStyles(styleRefs);
    const theme = useTheme();
    const { ruleManager } = useContext(StyleInjectorContext);
    const ownRuleUsageCountsByClassName = useRef({})
        .current;
    // Remove dynamic styles which are not used anymore when unmounting
    useEffect(() => () => {
        // eslint-disable-next-line guard-for-in, no-restricted-syntax
        for (const className in ownRuleUsageCountsByClassName) {
            const usageCount = ownRuleUsageCountsByClassName[className];
            ruleManager.decreaseUsage(className, usageCount);
        }
    }, 
    // eslint-disable-next-line react-hooks/exhaustive-deps
    []);
    return function sx(themedStyle) {
        let className = '';
        // eslint-disable-next-line guard-for-in, no-restricted-syntax
        for (const alias in themedStyle) {
            const value = themedStyle[alias];
            const shorthand = theme.aliases[alias] || alias;
            // eslint-disable-next-line no-loop-func
            (theme.shorthands[shorthand] || [shorthand]).forEach((key) => {
                // TODO: Support selectors and media queries
                if (typeof value !== 'object') {
                    const identName = `${key}-${value}`;
                    let appendedClassName = staticClassNames[identName];
                    // Attach a class dynamically if needed
                    if (!appendedClassName) {
                        // TODO: Use same hashing algorithm during static CSS generation
                        appendedClassName = getClassName(identName);
                        ruleManager.increaseUsage(appendedClassName, () => `.${appendedClassName}{${
                        // TODO: Abstract this logic away to a utility function
                        // Convert CSS property to kebab-case and normalize numeric value
                        `${key.replace(/[A-Z]/g, kebabCaseReplacer)}:${typeof value !== 'number' ? value : `${value}px`}`}}`);
                        ownRuleUsageCountsByClassName[appendedClassName] =
                            // eslint-disable-next-line no-bitwise
                            (ownRuleUsageCountsByClassName[appendedClassName] | 0) + 1;
                    }
                    className += ` ${appendedClassName}`;
                }
            });
        }
        // Remove prepended whitespace
        return className.slice(1);
    };
}

export { StyleInjectorProvider, ThemeProvider, VirtualStyleInjector, createTheme, defaultTheme, modularScale, setInstance as setLogger, useStyling, useTheme };
//# sourceMappingURL=index.js.map
